import { Request, Response } from 'express'
import { CreateDriverUseCase } from '@/libs/application/use-cases/driver/create.use-case'
import { UpdateDriverUseCase } from '@/libs/application/use-cases/driver/update.use-case'
import { DeleteDriverUseCase } from '@/libs/application/use-cases/driver/delete.use-case'
import { ShowDriverUseCase } from '@/libs/application/use-cases/driver/show.use-case'
import { ListDriversUseCase } from '@/libs/application/use-cases/driver/list.use-case'
import { Filters } from '@/libs/domain/criteria/filters'
import { OffsetPagination } from '@/libs/domain/criteria/offset-pagination'
import { Order } from '@/libs/domain/criteria/order'
import { OffsetCriteria } from '@/libs/domain/criteria/criteria'
import { RequestContext } from '@/libs/domain/contexts/request.context'
import { container } from '../di/container'

const createDriverUseCase = container.get(CreateDriverUseCase)
const updateDriverUseCase = container.get(UpdateDriverUseCase)
const deleteDriverUseCase = container.get(DeleteDriverUseCase)
const showDriverUseCase = container.get(ShowDriverUseCase)
const listDriversUseCase = container.get(ListDriversUseCase)

export class DriverController {
  static async create(req: Request, res: Response) {
    const driver = await createDriverUseCase.run(req.body, RequestContext.fromPrimitives(req))

    return res.status(201).json(driver)
  }

  static async update(req: Request, res: Response) {
    const { id } = req.params

    const driver = await updateDriverUseCase.run(id, req.body, RequestContext.fromPrimitives(req))

    return res.status(200).json(driver)
  }

  static async delete(req: Request, res: Response) {
    const { id } = req.params

    await deleteDriverUseCase.run(id, RequestContext.fromPrimitives(req))

    return res.sendStatus(204)
  }

  static async show(req: Request, res: Response) {
    const { id } = req.params

    const driver = await showDriverUseCase.run(id, RequestContext.fromPrimitives(req))

    return res.status(200).json(driver)
  }

  static async list(req: Request, res: Response) {
    const filters = Filters.fromPrimitives(req.query, [
      'user.firstName',
      'user.lastName',
      'user.email',
      'user.phoneNumber',
    ])
    const pagination = OffsetPagination.fromPrimitives(req.query)
    const order = Order.fromPrimitives(req.query, ['user.firstName', 'user.lastName', 'createdAt'])

    const criteria = new OffsetCriteria({
      filters,
      pagination,
      order,
    })

    const response = await listDriversUseCase.run(criteria, RequestContext.fromPrimitives(req))

    return res.status(200).json(response)
  }
}
