import { Request, Response } from 'express'
import { ListOrdersUseCase } from '@/libs/application/use-cases/order/list.use-case'
import { ShowOrderUseCase } from '@/libs/application/use-cases/order/show.use-case'
import { CreateOrderUseCase } from '@/libs/application/use-cases/order/create.use-case'
import { RequestContext } from '@/libs/domain/contexts/request.context'
import { OffsetCriteria } from '@/libs/domain/criteria/criteria'
import { Filters } from '@/libs/domain/criteria/filters'
import { OffsetPagination } from '@/libs/domain/criteria/offset-pagination'
import { Order } from '@/libs/domain/criteria/order'
import { container } from '../di/container'

const createOrderUseCase = container.get(CreateOrderUseCase)
const listOrdersUseCase = container.get(ListOrdersUseCase)
const showOrderUseCase = container.get(ShowOrderUseCase)

export class OrderController {
  static async create(req: Request, res: Response) {
    const response = await createOrderUseCase.run(req.body, RequestContext.fromPrimitives(req))
    return res.status(201).json(response)
  }

  static async show(req: Request, res: Response) {
    const { id } = req.params
    const response = await showOrderUseCase.run(id, RequestContext.fromPrimitives(req))
    return res.json(response)
  }

  static async list(req: Request, res: Response) {
    const filters = Filters.fromPrimitives(req.query, [
      'price',
      'status',
      'declineReason',
      'orderQuote.code',
    ])
    const pagination = OffsetPagination.fromPrimitives(req.query)
    const order = Order.fromPrimitives(req.query, ['price', 'createdAt'])

    const criteria = new OffsetCriteria({
      filters,
      pagination,
      order,
    })
    const response = await listOrdersUseCase.run(criteria, RequestContext.fromPrimitives(req))
    return res.json(response)
  }
}
