import { Given, Then, When } from '@cucumber/cucumber'
import { CustomWorld } from '../custom-world'
import { expect } from 'chai'
import { MailHelper } from '../helpers/mail-helper'
import {
  testAdminEmployee,
  testCustomer,
  testDriver,
  testSuperAdminUser,
  testVendorEmployee,
} from '@/libs/infrastructure/persistance/seeders/dev.seeder'

import { Response } from 'supertest'

const getCredentials = (role: string): { rut?: string; password?: string; email?: string } => {
  if (role === 'admin employee') {
    return {
      rut: testAdminEmployee.rut!,
      password: testAdminEmployee.password!,
    }
  }
  if (role === 'vendor employee') {
    return {
      rut: testVendorEmployee.rut!,
      password: testVendorEmployee.password!,
    }
  }
  if (role === 'customer') {
    return {
      rut: testCustomer.rut!,
      password: testCustomer.password!,
    }
  }
  if (role === 'driver') {
    return {
      rut: testDriver.rut!,
      password: testDriver.password!,
    }
  }

  if (role === 'super admin') {
    return {
      email: testSuperAdminUser.email!,
      password: testSuperAdminUser.password!,
    }
  }

  throw new Error(`Role "${role}" doesn't exists.`)
}

Given('I log in as {string}', async function (this: CustomWorld, role: string) {
  let response: Response

  if (role === 'super admin') {
    response = await this.requestApi.post('/admin/auth/sign-in').send(getCredentials(role))
  } else {
    response = await this.requestApi
      .post('/auth/sign-in')
      .set('x-tenantsubdomain', this.tenantSubdomain)
      .send(getCredentials(role))
  }

  expect(response.body).to.have.property('accessToken')
  this.accessToken = response.body.accessToken
})

Then('I get a status code {int}', async function (this: CustomWorld, statusCode: number) {
  if (![400, 500].includes(statusCode) && [400, 500].includes(this.response?.status!)) {
    // eslint-disable-next-line no-console
    console.log(this.response?.body)
  }
  expect(this.response!.status).to.be.equal(statusCode)
  if ([200, 201, 204].includes(statusCode)) {
    // eslint-disable-next-line no-unused-expressions
    expect(this.response!.error).to.be.false
  }
})

Then('I get a response with message {string}', async function (this: CustomWorld, message: string) {
  expect(this.response!.body.message).to.be.equal(message)
})

When(
  'I send a POST request to {string} with body:',
  async function (this: CustomWorld, path: string, body: string) {
    this.response = await this.requestApi
      .post(path)
      .set('x-tenantsubdomain', this.tenantSubdomain)
      .set('Authorization', `Bearer ${this.accessToken}`)
      .send(JSON.parse(body))
  }
)

When('I send a POST request to {string}', async function (this: CustomWorld, path: string) {
  this.response = await this.requestApi
    .post(path)
    .set('x-tenantsubdomain', this.tenantSubdomain)
    .set('Authorization', `Bearer ${this.accessToken}`)
    .send()
})

When('I send a GET request to {string}', async function (this: CustomWorld, path: string) {
  this.response = await this.requestApi
    .get(path)
    .set('x-tenantsubdomain', this.tenantSubdomain)
    .set('Authorization', `Bearer ${this.accessToken}`)
})

When(
  'I send a PATCH request to {string} with body:',
  async function (this: CustomWorld, path: string, body: string) {
    this.response = await this.requestApi
      .patch(path)
      .set('x-tenantsubdomain', this.tenantSubdomain)
      .set('Authorization', `Bearer ${this.accessToken}`)
      .send(JSON.parse(body))
  }
)

When('I send a DELETE request to {string}', async function (this: CustomWorld, path: string) {
  this.response = await this.requestApi
    .delete(path)
    .set('x-tenantsubdomain', this.tenantSubdomain)
    .set('Authorization', `Bearer ${this.accessToken}`)
})

Then('I get a response body with:', async function (this: CustomWorld, responseBody: string) {
  expect(this.response?.body).containSubset(JSON.parse(responseBody))
})

Then(
  'I get a response body with a field {string}',
  async function (this: CustomWorld, responseBodyField: string) {
    expect(this.response?.body).to.have.property(responseBodyField)
  }
)

Then('I get a reset link to {string}', async function (this: CustomWorld, email: string) {
  const resetToken = await MailHelper.getResetTokenFromEmail(email)

  expect(resetToken).to.be.a('string')
})
