import { Injectable } from '@/core/api/di/decorators'
import { RequestContext } from '@/libs/domain/contexts/request.context'
import { FormTemplateEntity } from '@/libs/domain/entities/tenant/form-template.entity'
import { BadRequestException, NotFoundException } from '@/libs/domain/errors'
import { FormTemplateRepository } from '@/libs/domain/repositories/tenant/form-template.repository'

import { v4 as uuidv4 } from 'uuid'

@Injectable()
export class UpdateFormTemplateUseCase {
  constructor(private readonly formTemplateRepository: FormTemplateRepository) {}

  async run(
    payload: Pick<FormTemplateEntity, 'fields'>,
    { tenant }: RequestContext
  ): Promise<FormTemplateEntity> {
    const formTemplate = await this.formTemplateRepository.findOne({}, tenant.id)
    if (!formTemplate) {
      throw new NotFoundException('Plantilla de formulario no encontrada.')
    }

    const updatedDefaultFields = formTemplate.fields
      .filter(field => field.isDefault)
      .map(field => {
        const updatedDefaultField = payload.fields.find(({ id }) => id === field.id)
        if (!updatedDefaultField) {
          throw new BadRequestException(`Campo ${field.label} no puede ser eliminado.`)
        }

        return { ...field, order: updatedDefaultField.order, span: updatedDefaultField.span }
      })

    const otherFields = payload.fields
      .filter(field => !field.isDefault)
      .map(field => {
        return { ...field, id: field.id ? field.id : uuidv4(), isDefault: false }
      })

    return await this.formTemplateRepository.save(
      {
        id: formTemplate.id,
        fields: [...updatedDefaultFields, ...otherFields],
      },
      tenant.id
    )
  }
}
