/* eslint-disable sort-exports/sort-exports */
import {
  And,
  FindManyOptions,
  ILike,
  In,
  IsNull,
  LessThan,
  LessThanOrEqual,
  MoreThan,
  MoreThanOrEqual,
  Not,
  ObjectLiteral,
  FindOneOptions as TypeOrmFindOneOptions,
} from 'typeorm'

import _get from 'lodash/get'
import _set from 'lodash/set'

import { CursorCriteria, OffsetCriteria } from '@/libs/domain/criteria/criteria'
import { Filter, FilterOperator } from '@/libs/domain/criteria/filters'

export type TypeOrmOptions = {
  order?: { [key: string]: 'asc' | 'desc' }
  where?: { [key: string]: any }
  take?: number
  skip?: number
}

export class TypeOrmCriteriaParser {
  static async parseCursorCriteria(criteria: CursorCriteria) {
    const query: TypeOrmOptions = {}
    if (criteria.hasFilters()) {
      query.where = criteria.filters.value.reduce((where, filter) => {
        return this.generateWhereCondition(filter, where)
      }, {})
    }

    if (criteria.hasCursor()) {
      const payload = await criteria.pagination.decodeCursor()

      query.where = this.generateWhereCondition(
        {
          field: criteria.pagination.field,
          operator: criteria.pagination.orderType === 'asc' ? FilterOperator.GT : FilterOperator.LT,
          value: payload[criteria.pagination.field],
        },
        query.where || {}
      )
    }

    query.order = {
      [criteria.pagination.field]: criteria.pagination.orderType,
    }

    query.take = criteria.pagination.pageSize

    return query
  }

  static async parseOffsetCriteria(criteria: OffsetCriteria) {
    const query: TypeOrmOptions = {}
    if (criteria.hasFilters()) {
      query.where = criteria.filters.value.reduce((where, filter) => {
        return this.generateWhereCondition(filter, where)
      }, {})
    }

    if (criteria.hasOrder()) {
      query.order = _set({}, criteria.order!.orderBy!, criteria.order!.orderType)
    }

    query.take = criteria.pagination.pageSize
    query.skip = criteria.pagination.pageSize * (criteria.pagination.page - 1)

    return query
  }

  private static generateWhereCondition(filter: Filter, { ...where }: { [key: string]: any }) {
    const mappingFilterValue = {
      [FilterOperator.EQUAL]: filter.value,
      [FilterOperator.NOT_EQUAL]: Not(filter.value),

      [FilterOperator.CONTAINS]: ILike(`%${filter.value}%`),
      [FilterOperator.NOT_CONTAINS]: Not(ILike(`%${filter.value}%`)),

      [FilterOperator.GT]: MoreThan(filter.value),
      [FilterOperator.GTE]: MoreThanOrEqual(filter.value),
      [FilterOperator.LT]: LessThan(filter.value),
      [FilterOperator.LTE]: LessThanOrEqual(filter.value),

      [FilterOperator.IS_NULL]: IsNull(),
      [FilterOperator.IS_NOT_NULL]: Not(IsNull()),

      [FilterOperator.ANY_OF]: In(filter.value as any[]),
    }

    const previousValue = _get(where, filter.field)

    return _set(
      where,
      filter.field,
      previousValue
        ? And(previousValue as any, mappingFilterValue[filter.operator] as any)
        : mappingFilterValue[filter.operator]
    )
  }
}

export type FindOneOptions<Schema extends ObjectLiteral> = Omit<
  TypeOrmFindOneOptions<Schema>,
  'where' | 'relations' | 'order'
>

export type Relations<Schema extends ObjectLiteral> = TypeOrmFindOneOptions<Schema>['relations']

export type FindOptions<Schema extends ObjectLiteral> = Omit<
  FindManyOptions<Schema>,
  'where' | 'relations' | 'take' | 'skip' | 'select'
>

export type FindCriteriaOptions<Schema extends ObjectLiteral> = Omit<
  FindManyOptions<Schema>,
  'where' | 'relations' | 'take' | 'order' | 'skip' | 'select'
>
